//  SpinningEarth-D3D.hpp

#pragma once

#include "Macros-D3D.h"

//  REFINEMENT_LEVEL tells how finely the triangulation will
//  be subdivided.
//
//      If REFINEMENT_LEVEL is 0 the Earth will be
//          triangulated as a plain icosahedron (20 faces).
//
//      If REFINEMENT_LEVEL is 1 the Earth will be
//          triangulated as the first subdivision (80 faces).
//
//      If REFINEMENT_LEVEL is 2 the Earth will be
//          triangulated as the second subdivision (320 faces).
//
//      etc.
#define REFINEMENT_LEVEL    2


class CSpinningEarth
{
public:

                    CSpinningEarth(void);
    virtual         ~CSpinningEarth(void);

    virtual void    Update(double anElapsedTime);
    virtual void    Render( LPDIRECT3DDEVICE7       aDevice,
                            D3DXMATRIX              *aPosition,
                            D3DXMATRIX              *aCameraPosition,
                            LPDIRECTDRAWSURFACE7    aTexture,
                            DWORD                   aColor,
                            int                     aFogRange,
                            DWORD                   (*aFogFunction)(float *, float *, int, float *));

protected:
    double      itsRotationAngle;

    //  Data for the icosahedral subdivision of the sphere.
    D3DLVERTEX  *itsVertices[REFINEMENT_LEVEL + 1];
    WORD        *itsFaces[REFINEMENT_LEVEL + 1];
    int         itsNumVertices[REFINEMENT_LEVEL + 1],
                itsNumFaces[REFINEMENT_LEVEL + 1];

    void        InitIcosahedron(void);
    void        ComputeTextureCoordinates(float x, float y, float z, D3DLVERTEX *aVertex);
    void        SubdivideTriangulation(int aLevel);
};
