//  SpinningEarth-GL.hpp

#pragma once

#include "Macros-GL.h"

//  REFINEMENT_LEVEL tells how finely the triangulation will
//  be subdivided.
//
//      If REFINEMENT_LEVEL is 0 the Earth will be
//          triangulated as a plain octahedron (8 faces).
//
//      If REFINEMENT_LEVEL is 1 the Earth will be
//          triangulated as the first subdivision (32 faces).
//
//      If REFINEMENT_LEVEL is 2 the Earth will be
//          triangulated as the second subdivision (128 faces).
//
//      etc.
#define REFINEMENT_LEVEL    3

typedef GLdouble    Vertex[3],
                    TextureUV[2];

class CSpinningEarth
{
public:

                    CSpinningEarth(void);
    virtual         ~CSpinningEarth(void);

    virtual void    Update(double anElapsedTime);

    virtual void    MakeDisplayLists(void);
    virtual void    FreeDisplayLists(void);
    virtual void    RenderDisplayList(GLuint aTexture, float aDistance);

protected:
    double      itsRotationAngle;
    GLuint      itsDisplayLists[REFINEMENT_LEVEL + 1];

    //  Data for the octahedral subdivision of the sphere.
    Vertex      *itsVertices[REFINEMENT_LEVEL + 1];
    TextureUV   *itsTextureCoordinates[REFINEMENT_LEVEL + 1];
    GLuint      *itsFaces[REFINEMENT_LEVEL + 1];
    int         itsNumVertices[REFINEMENT_LEVEL + 1],
                itsNumFaces[REFINEMENT_LEVEL + 1];

    void        InitOctahedron(void);
    void        SubdivideTriangulation(int aLevel);
    void        ComputeTextureCoordinates(Vertex v, TextureUV t);
    void        RemoveGap(int aLevel);
};
